/*
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.cloud.kms.v1;

import com.google.api.pathtemplate.PathTemplate;
import com.google.api.resourcenames.ResourceName;
import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import javax.annotation.Generated;

// AUTO-GENERATED DOCUMENTATION AND CLASS.
@Generated("by gapic-generator-java")
public class EkmConnectionName implements ResourceName {
  private static final PathTemplate PROJECT_LOCATION_EKM_CONNECTION =
      PathTemplate.createWithoutUrlEncoding(
          "projects/{project}/locations/{location}/ekmConnections/{ekm_connection}");
  private volatile Map<String, String> fieldValuesMap;
  private final String project;
  private final String location;
  private final String ekmConnection;

  @Deprecated
  protected EkmConnectionName() {
    project = null;
    location = null;
    ekmConnection = null;
  }

  private EkmConnectionName(Builder builder) {
    project = Preconditions.checkNotNull(builder.getProject());
    location = Preconditions.checkNotNull(builder.getLocation());
    ekmConnection = Preconditions.checkNotNull(builder.getEkmConnection());
  }

  public String getProject() {
    return project;
  }

  public String getLocation() {
    return location;
  }

  public String getEkmConnection() {
    return ekmConnection;
  }

  public static Builder newBuilder() {
    return new Builder();
  }

  public Builder toBuilder() {
    return new Builder(this);
  }

  public static EkmConnectionName of(String project, String location, String ekmConnection) {
    return newBuilder()
        .setProject(project)
        .setLocation(location)
        .setEkmConnection(ekmConnection)
        .build();
  }

  public static String format(String project, String location, String ekmConnection) {
    return newBuilder()
        .setProject(project)
        .setLocation(location)
        .setEkmConnection(ekmConnection)
        .build()
        .toString();
  }

  public static EkmConnectionName parse(String formattedString) {
    if (formattedString.isEmpty()) {
      return null;
    }
    Map<String, String> matchMap =
        PROJECT_LOCATION_EKM_CONNECTION.validatedMatch(
            formattedString, "EkmConnectionName.parse: formattedString not in valid format");
    return of(matchMap.get("project"), matchMap.get("location"), matchMap.get("ekm_connection"));
  }

  public static List<EkmConnectionName> parseList(List<String> formattedStrings) {
    List<EkmConnectionName> list = new ArrayList<>(formattedStrings.size());
    for (String formattedString : formattedStrings) {
      list.add(parse(formattedString));
    }
    return list;
  }

  public static List<String> toStringList(List<EkmConnectionName> values) {
    List<String> list = new ArrayList<>(values.size());
    for (EkmConnectionName value : values) {
      if (value == null) {
        list.add("");
      } else {
        list.add(value.toString());
      }
    }
    return list;
  }

  public static boolean isParsableFrom(String formattedString) {
    return PROJECT_LOCATION_EKM_CONNECTION.matches(formattedString);
  }

  @Override
  public Map<String, String> getFieldValuesMap() {
    if (fieldValuesMap == null) {
      synchronized (this) {
        if (fieldValuesMap == null) {
          ImmutableMap.Builder<String, String> fieldMapBuilder = ImmutableMap.builder();
          if (project != null) {
            fieldMapBuilder.put("project", project);
          }
          if (location != null) {
            fieldMapBuilder.put("location", location);
          }
          if (ekmConnection != null) {
            fieldMapBuilder.put("ekm_connection", ekmConnection);
          }
          fieldValuesMap = fieldMapBuilder.build();
        }
      }
    }
    return fieldValuesMap;
  }

  public String getFieldValue(String fieldName) {
    return getFieldValuesMap().get(fieldName);
  }

  @Override
  public String toString() {
    return PROJECT_LOCATION_EKM_CONNECTION.instantiate(
        "project", project, "location", location, "ekm_connection", ekmConnection);
  }

  @Override
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (o != null || getClass() == o.getClass()) {
      EkmConnectionName that = ((EkmConnectionName) o);
      return Objects.equals(this.project, that.project)
          && Objects.equals(this.location, that.location)
          && Objects.equals(this.ekmConnection, that.ekmConnection);
    }
    return false;
  }

  @Override
  public int hashCode() {
    int h = 1;
    h *= 1000003;
    h ^= Objects.hashCode(project);
    h *= 1000003;
    h ^= Objects.hashCode(location);
    h *= 1000003;
    h ^= Objects.hashCode(ekmConnection);
    return h;
  }

  /** Builder for projects/{project}/locations/{location}/ekmConnections/{ekm_connection}. */
  public static class Builder {
    private String project;
    private String location;
    private String ekmConnection;

    protected Builder() {}

    public String getProject() {
      return project;
    }

    public String getLocation() {
      return location;
    }

    public String getEkmConnection() {
      return ekmConnection;
    }

    public Builder setProject(String project) {
      this.project = project;
      return this;
    }

    public Builder setLocation(String location) {
      this.location = location;
      return this;
    }

    public Builder setEkmConnection(String ekmConnection) {
      this.ekmConnection = ekmConnection;
      return this;
    }

    private Builder(EkmConnectionName ekmConnectionName) {
      this.project = ekmConnectionName.project;
      this.location = ekmConnectionName.location;
      this.ekmConnection = ekmConnectionName.ekmConnection;
    }

    public EkmConnectionName build() {
      return new EkmConnectionName(this);
    }
  }
}
